@@ -4,6 +4,7 @@ import warnings as pywarnings
 from time import time
 from collections import defaultdict
 import numpy as np
+from ..nomials.map import NomialMap
 from ..repr_conventions import lineagestr
 from ..small_classes import CootMatrix, SolverLog, Numbers, FixedScalar
 from ..small_scripts import appendsolwarning, initsolwarning
@@ -213,7 +214,7 @@ class GeometricProgram:
         except Exception as e:
             raise UnknownInfeasible("Something unexpected went wrong.") from e
         finally:
-            self.solve_log = "\n".join(sys.stdout)
+            self.solve_log = sys.stdout
             sys.stdout = original_stdout
             self.solver_out = solver_out
 
@@ -307,7 +308,8 @@ class GeometricProgram:
         return la, nu_by_posy
 
     def _compile_result(self, solver_out):
-        result = {"cost": float(solver_out["objective"])}
+        result = {"cost": float(solver_out["objective"]),
+                  "cost function": self.cost}
         primal = solver_out["primal"]
         if len(self.varlocs) != len(primal):
             raise RuntimeWarning("The primal solution was not returned.")
@@ -334,7 +336,7 @@ class GeometricProgram:
                 " %s, but since the '%s' solver doesn't support discretization"
                 " they were treated as continuous variables."
                 % (sorted(self.choicevaridxs.keys()), solver_out["solver"]),
-                self.choicevaridxs)]}
+                self.choicevaridxs)]}  # TODO: choicevaridxs seems unnecessary
 
         result["sensitivities"] = {"constraints": {}}
         la, self.nu_by_posy = self._generate_nula(solver_out)
@@ -342,36 +344,45 @@ class GeometricProgram:
                                                          self.cost.hmap))
         gpv_ss = cost_senss.copy()
         m_senss = defaultdict(float)
+        absv_ss = {vk: abs(x) for vk, x in cost_senss.items()}
         for las, nus, c in zip(la[1:], self.nu_by_posy[1:], self.hmaps[1:]):
             while getattr(c, "parent", None) is not None:
-                c = c.parent
+                if not isinstance(c, NomialMap):
+                    c.parent.child = c
+                c = c.parent  # parents get their sens_from_dual used...
             v_ss, c_senss = c.sens_from_dual(las, nus, result)
             for vk, x in v_ss.items():
                 gpv_ss[vk] = x + gpv_ss.get(vk, 0)
+                absv_ss[vk] = abs(x) + absv_ss.get(vk, 0)
             while getattr(c, "generated_by", None):
-                c = c.generated_by
-            result["sensitivities"]["constraints"][c] = abs(c_senss)
+                c.generated_by.generated = c
+                c = c.generated_by  # ...while generated_bys are just labels
+            result["sensitivities"]["constraints"][c] = c_senss
             m_senss[lineagestr(c)] += abs(c_senss)
-        # add fixed variables sensitivities to models
-        for vk, senss in gpv_ss.items():
-            m_senss[lineagestr(vk)] += abs(senss)
         result["sensitivities"]["models"] = dict(m_senss)
         # carry linked sensitivities over to their constants
         for v in list(v for v in gpv_ss if v.gradients):
             dlogcost_dlogv = gpv_ss.pop(v)
+            dlogcost_dlogabsv = absv_ss.pop(v)
             val = np.array(result["constants"][v])
             for c, dv_dc in v.gradients.items():
                 with pywarnings.catch_warnings():  # skip pesky divide-by-zeros
                     pywarnings.simplefilter("ignore")
                     dlogv_dlogc = dv_dc * result["constants"][c]/val
                     gpv_ss[c] = gpv_ss.get(c, 0) + dlogcost_dlogv*dlogv_dlogc
+                    absv_ss[c] = (absv_ss.get(c, 0)
+                                  + abs(dlogcost_dlogabsv*dlogv_dlogc))
                 if v in cost_senss:
-                    if c in self.cost.vks:
+                    if c in self.cost.vks:  # TODO: seems unnecessary
                         dlogcost_dlogv = cost_senss.pop(v)
                         before = cost_senss.get(c, 0)
                         cost_senss[c] = before + dlogcost_dlogv*dlogv_dlogc
+        # add fixed variables sensitivities to models
+        for vk, senss in gpv_ss.items():
+            m_senss[lineagestr(vk)] += abs(senss)
         result["sensitivities"]["cost"] = cost_senss
         result["sensitivities"]["variables"] = KeyDict(gpv_ss)
+        result["sensitivities"]["variablerisk"] = KeyDict(absv_ss)
         result["sensitivities"]["constants"] = \
             result["sensitivities"]["variables"]  # NOTE: backwards compat.
         return SolutionArray(result)
