@@ -1,6 +1,6 @@
 "Implements Loose"
 from .set import ConstraintSet
-from ..small_scripts import appendsolwarning
+from ..small_scripts import appendsolwarning, initsolwarning
 
 
 class Loose(ConstraintSet):
@@ -15,6 +15,7 @@ class Loose(ConstraintSet):
     def process_result(self, result):
         "Checks that all constraints are satisfied with equality"
         super().process_result(result)
+        initsolwarning(result, "Unexpectedly Tight Constraints")
         for constraint in self.flat():
             c_senss = result["sensitivities"]["constraints"].get(constraint, 0)
             if c_senss >= self.senstol:
@@ -23,8 +24,7 @@ class Loose(ConstraintSet):
                 msg = ("%s is not loose: it has a sensitivity of %+.4g."
                        " (Allowable sensitivity: %.4g)" %
                        (cstr, c_senss, self.senstol))
-                constraint.relax_sensitivity = c_senss
-                appendsolwarning(msg, constraint, result,
+                appendsolwarning(msg, (c_senss, constraint), result,
                                  "Unexpectedly Tight Constraints")
                 if self.raiseerror:
                     raise RuntimeWarning(msg)
