@@ -3,6 +3,7 @@ from .set import ConstraintSet
 from ..nomials import Variable, VectorVariable, parse_subs, NomialArray
 from ..keydict import KeyDict
 from .. import NamedVariables, SignomialsEnabled
+from ..small_scripts import appendsolwarning, initsolwarning, mag
 
 
 class ConstraintsRelaxedEqually(ConstraintSet):
@@ -39,8 +40,21 @@ class ConstraintsRelaxedEqually(ConstraintSet):
                                    for c in original_constraints.flat()]
 
         ConstraintSet.__init__(self, {
-            "relaxed constraints": relaxed_constraints,
-            "minimum relaxation": self.relaxvar >= 1}, original_substitutions)
+            "minimum relaxation": self.relaxvar >= 1,
+            "relaxed constraints": relaxed_constraints}, original_substitutions)
+
+    def process_result(self, result):
+        "Warns if any constraints were relaxed"
+        super().process_result(result)
+        self.check_relaxed(result)
+
+    def check_relaxed(self, result):
+        "Adds relaxation warnings to the result"
+        initsolwarning(result, "Relaxed Constraints")
+        for val, msg in get_relaxed([result["freevariables"][self.relaxvar]],
+                                    ["All constraints relaxed by %i%%"]):
+            appendsolwarning(msg % (0.9+(val-1)*100), self, result,
+                             "Relaxed Constraints")
 
 
 class ConstraintsRelaxed(ConstraintSet):
@@ -78,8 +92,39 @@ class ConstraintsRelaxed(ConstraintSet):
                 for i, c in enumerate(original_constraints.flat())]
 
         ConstraintSet.__init__(self, {
-            "relaxed constraints": relaxed_constraints,
-            "minimum relaxation": self.relaxvars >= 1}, original_substitutions)
+            "minimum relaxation": self.relaxvars >= 1,
+            "relaxed constraints": relaxed_constraints}, original_substitutions)
+
+    def process_result(self, result):
+        "Warns if any constraints were relaxed"
+        super().process_result(result)
+        self.check_relaxed(result)
+
+    def check_relaxed(self, result):
+        "Adds relaxation warnings to the result"
+        relaxed = get_relaxed(result["freevariables"][self.relaxvars],
+                              range(len(self["relaxed constraints"])))
+        initsolwarning(result, "Relaxed Constraints")
+        for relaxval, i in relaxed:
+            relax_percent = "%i%%" % (0.5+(relaxval-1)*100)
+            oldconstraint = self.original_constraints[i]
+            newconstraint = self["relaxed constraints"][i][0]
+            subs = {self.relaxvars[i]: relaxval}
+            relaxdleft = newconstraint.left.sub(subs)
+            relaxdright = newconstraint.right.sub(subs)
+            oldleftstr = str(oldconstraint.left)
+            relaxedleftstr = str(relaxdleft)
+            padding = len(relaxedleftstr) - len(oldleftstr)
+            if padding > 0:
+                oldleftstr = " " * padding + oldleftstr
+            elif padding < 0:
+                relaxedleftstr = " " * padding + relaxedleftstr
+            msg = (" %3i: %5s relaxed, from %s %s %s\n"
+                   "                       to %s %s %s"
+                   % (i, relax_percent, oldleftstr,
+                      oldconstraint.oper, oldconstraint.right,
+                      relaxedleftstr, newconstraint.oper, relaxdright))
+            appendsolwarning(msg, oldconstraint, result, "Relaxed Constraints")
 
 
 class ConstantsRelaxed(ConstraintSet):
@@ -117,7 +162,7 @@ class ConstantsRelaxed(ConstraintSet):
         if not isinstance(constraints, ConstraintSet):
             constraints = ConstraintSet(constraints)
         substitutions = KeyDict(constraints.substitutions)
-        constants, _, linked = parse_subs(constraints.varkeys, substitutions)
+        constants, _, linked = parse_subs(constraints.vks, substitutions)
         if linked:
             kdc = KeyDict(constants)
             constrained_varkeys = constraints.constrained_varkeys()
@@ -168,9 +213,33 @@ class ConstantsRelaxed(ConstraintSet):
         self.constants = constants
 
     def process_result(self, result):
-        "Transfers the constant sensitivities back to the original constants"
-        ConstraintSet.process_result(self, result)
+        "Transfers constant sensitivities back to the original constants"
+        super().process_result(result)
         constant_senss = result["sensitivities"]["variables"]
         for new_constant, former_constant in self._derelax_map.items():
             constant_senss[former_constant] = constant_senss[new_constant]
             del constant_senss[new_constant]
+        self.check_relaxed(result)
+
+
+    def check_relaxed(self, result):
+        "Adds relaxation warnings to the result"
+        relaxed = get_relaxed([result["freevariables"][r]
+                               for r in self.relaxvars], self.freedvars)
+        initsolwarning(result, "Relaxed Constants")
+        for (_, freed) in relaxed:
+            msg = ("  %s: relaxed from %-.4g to %-.4g"
+                   % (freed,
+                      mag(self.constants[freed.key]),
+                      mag(result["freevariables"][freed])))
+            appendsolwarning(msg, freed, result, "Relaxed Constants")
+
+
+def get_relaxed(relaxvals, mapped_list):
+    "Returns 'relaxed' vals (those above an arbitrary threshold of 1.01)."
+    sortrelaxed = sorted(zip(relaxvals, mapped_list), key=lambda x: -x[0])
+    mostrelaxed = max(sortrelaxed[0][0], 1.01)
+    for i, (val, _) in enumerate(sortrelaxed):
+        if val <= 1.01 and (val-1) <= (mostrelaxed-1)/10:
+            return sortrelaxed[:i]
+    return sortrelaxed
